package com.sundata.internalevaluation.configuration.controller;

import com.sundata.common.base.BaseAction;
import com.sundata.common.exception.BusinessException;
import com.sundata.common.exception.ServiceException;
import com.sundata.internalevaluation.configuration.model.DataSetConfigModel;
import com.sundata.internalevaluation.configuration.model.DataSetConfigModel;
import com.sundata.internalevaluation.configuration.model.DataSourcesModel;
import com.sundata.internalevaluation.configuration.service.DataSetConfigService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/internalevaluation/configuration/DataSetConfigAction")
public class DataSetConfigAction extends BaseAction {

    static final Logger log = LoggerFactory.getLogger(DataSetConfigAction.class);

    @Autowired
    DataSetConfigService dataSetConfigService;


    /**
     * @author: 张艳杰
     * @methodsName: getDataList
     * @description: 数据集配置初始化
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody DataSetConfigModel model
     * @return: List<DataSetConfigModel>
     * @throws:
     */
    @RequestMapping({"getDataList"})
    public List<DataSetConfigModel> getDataList(HttpServletRequest request, HttpServletResponse response,
                                              @RequestBody DataSetConfigModel model) {
        return dataSetConfigService.getDataList(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: createData
     * @description: 数据集配置新增
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody DataSetConfigModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"createData"})
    public void createData(HttpServletRequest request, HttpServletResponse response,
                           @RequestBody DataSetConfigModel model) {
        dataSetConfigService.createData(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: deleteRows
     * @description: 数据集配置删除
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody DataSetConfigModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"deleteRows"})
    public void deleteRows(HttpServletRequest request, HttpServletResponse response,
                           @RequestBody DataSetConfigModel model) {
        dataSetConfigService.deleteRows(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: updateExistData
     * @description: 数据集配置修改
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody DataSetConfigModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"updateExistData"})
    public void updateExistData(HttpServletRequest request, HttpServletResponse response,
                                @RequestBody DataSetConfigModel model) {
        dataSetConfigService.updateExistData(model);
    }


    /**
     * @author: 张艳杰
     * @methodsName: selectDetailData
     * @description: 数据来源配置查询
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody DataSetConfigModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"selectDetailData"})
    public DataSetConfigModel selectDetailData(HttpServletRequest request, HttpServletResponse response,
                                             @RequestBody DataSetConfigModel model) {
        return dataSetConfigService.selectDetailData(model);
    }

}
