package com.sundata.internalevaluation.configuration.service;

import com.sundata.common.util.JsonUtil;
import com.sundata.internalevaluation.configuration.mapper.DataSourcesMapper;
import com.sundata.internalevaluation.configuration.model.DataItemConfigModel;
import com.sundata.internalevaluation.configuration.model.DataSourcesModel;
import com.sundata.internalevaluation.configuration.model.InputParamModel;
import com.sundata.internalevaluation.configuration.model.QueryLogicModel;
import org.bouncycastle.pqc.jcajce.provider.sphincsplus.SignatureSpi;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.annotation.AccessType;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class DataSourcesService {

    @Autowired
    DataSourcesMapper dataSourcesMapper;

    /**
     * @author: 张艳杰
     * @methodsName: getDataList
     * @description: 数据来源配置初始化
     * @param: DataSourcesModel model
     * @return: List<DataSourcesModel>
     * @throws:
     */
    public List<DataSourcesModel> getDataList(DataSourcesModel model) {
        return dataSourcesMapper.getDataList(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: createData
     * @description: 数据来源配置新增
     * @param: DataSourcesModel model
     * @return:
     * @throws:
     */
    public void createData(DataSourcesModel model) {
        model.setRequestInterface(JsonUtil.toJSONString(model.getRequestInterfaces()));
        dataSourcesMapper.createData(model);

        if (null != model.getInputParam() && model.getInputParam().size()>0) {
            dataSourcesMapper.createInputParamData(model.getInputParam());
        }

        if (null != model.getDataItemConfig() && model.getDataItemConfig().size()>0) {
            for (DataItemConfigModel m : model.getDataItemConfig()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            dataSourcesMapper.createDataItemConfigData(model.getDataItemConfig());
        }

        if (null != model.getQueryLogic() && model.getQueryLogic().size()>0) {
            for ( QueryLogicModel m : model.getQueryLogic()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            dataSourcesMapper.createQueryLogicData(model.getQueryLogic());
        }

    }

    /**
     * @author: 张艳杰
     * @methodsName: deleteRows
     * @description: 数据来源配置删除
     * @param: DataSourcesModel model
     * @return:
     * @throws:
     */
    public void deleteRows(DataSourcesModel model) {
        dataSourcesMapper.deleteRows(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: updateExistData
     * @description: 数据来源配置修改
     * @param: DataSourcesModel model
     * @return:
     * @throws:
     */
    public void updateExistData(DataSourcesModel model) {
        model.setRequestInterface(JsonUtil.toJSONString(model.getRequestInterfaces()));
        dataSourcesMapper.updateExistData(model);
        dataSourcesMapper.deleteInputParamRows();
        dataSourcesMapper.deleteDataSourceRows(model);
        dataSourcesMapper.deleteQueryLogicRows(model);

        if (null != model.getInputParam() && model.getInputParam().size()>0) {
            dataSourcesMapper.createInputParamData(model.getInputParam());
        }

        if (null != model.getDataItemConfig() && model.getDataItemConfig().size()>0) {
            for (DataItemConfigModel m : model.getDataItemConfig()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            dataSourcesMapper.createDataItemConfigData(model.getDataItemConfig());
        }

        if (null != model.getQueryLogic() && model.getQueryLogic().size()>0) {
            for ( QueryLogicModel m : model.getQueryLogic()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            dataSourcesMapper.createQueryLogicData(model.getQueryLogic());
        }


    }

    /**
     * @author: 张艳杰
     * @methodsName: selectRowData
     * @description: 数据来源配置修改
     * @param: DataSourcesModel model
     * @return:
     * @throws:
     */
    public DataSourcesModel selectDetailData(String dataSourcesNo) {
        DataSourcesModel model = dataSourcesMapper.selectDetailData(dataSourcesNo);
        model.setRequestInterfaces(JsonUtil.jsonToList(model.getRequestInterface()));
        List<InputParamModel> inputList = dataSourcesMapper.getInputParamList(dataSourcesNo);
        List<DataItemConfigModel> dataItemList = dataSourcesMapper.getDataItemConfigList(dataSourcesNo);
        List<QueryLogicModel> queryLogicList = dataSourcesMapper.getQueryLogicList(dataSourcesNo);
        model.setInputParam(inputList);
        model.setQueryLogic(queryLogicList);
        model.setDataItemConfig(dataItemList);
        return model;
    }



}
