package com.sundata.internalevaluation.script.pool;

import groovy.lang.GroovyShell;
import groovy.lang.Script;
import org.apache.commons.pool2.BasePooledObjectFactory;
import org.apache.commons.pool2.DestroyMode;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.PooledSoftReference;
import com.sundata.internalevaluation.script.model.ScriptModel;

import java.lang.ref.SoftReference;

public class GroovyScriptFactory extends BasePooledObjectFactory<Script> {
    private final GroovyShell groovyShell = new GroovyShell();
    private final ScriptModel scriptModel;

    public GroovyScriptFactory(ScriptModel scriptModel) {
        this.scriptModel = scriptModel;
    }
    /**
     * Creates an object instance, to be wrapped in a {@link PooledObject}.
     * <p>This method <strong>must</strong> support concurrent, multi-threaded
     * invocation.</p>
     *
     * @return an instance to be served by the pool, not null.
     */
    @Override
    public Script create() {
        return groovyShell.parse(scriptModel.getScriptStr());
    }

    /**
     * Wraps the provided instance with an implementation of
     * {@link PooledObject}.
     *
     * @param obj the instance to wrap, should not be null.
     * @return The provided instance, wrapped by a {@link PooledObject}
     */
    @Override
    public PooledObject<Script> wrap(Script obj) {
        return new PooledSoftReference<>(new SoftReference<>(obj));
    }

    /**
     * Destroys an instance no longer needed by the pool, using the provided
     * DestroyMode.
     *
     * @param p           a {@code PooledObject} wrapping the instance to be destroyed
     * @param destroyMode DestroyMode providing context to the factory
     * @throws Exception should be avoided as it may be swallowed by
     *                   the pool implementation.
     * @see #validateObject
     * @see #destroyObject(PooledObject)
     * @see DestroyMode
     * @since 2.9.0
     */
    @Override
    public void destroyObject(PooledObject<Script> p, DestroyMode destroyMode) throws Exception {
        super.destroyObject(p, destroyMode);
    }
}
